using namespace std;

#include "../stella/Console.h"
#include "../stella/Sound.h"
#include "../stella/MediaSrc.h"
#include "../stella/Event.h"
#include "../stella/StellaEvent.h"
#include "../stella/EventHandler.h"
#include "../stella/SoundDS.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <stdint.h>

#include <dingkit/fsys.h>
#include <dingkit/dingoo.h>
#include <dingkit/graphics.h>
#include <dingkit/display.h>
#include <dingkit/control.h>
#include <dingkit/timer.h>
#include <dingkit/fixmath.h>
#include <dingkit/audio.h>
#include <dingkit/keyboard.h>
#include <dingkit/ucos2.h>

#include "a320_snd.h"
#include "StelGOO.h"

Console* theConsole = NULL;
Sound* theSound = NULL;
uInt8* buffer_rom = NULL;
unsigned int buffer_rom_size = 0;
const uInt32* gamePalette;

char gameName[256];
gamecfg GameConf;

unsigned int m_Flag;

void (*render_screen)(int viswidth, int visheight, unsigned char *bmpline);

timer*   gameTimer    = NULL;
uint32_t gameTickRate = (TIMER_RESOLUTION / 30);

gfx_color gamePal[256];

void bitmap_noscale(int viswidth, int visheight, unsigned char *bmpline) {

  unsigned int inX,inY,inColor;
  uint16_t* buffer_scr = (unsigned short *)(gfx_render_target->address);
		
  for (inY=0;inY<SYSVID_HEIGHT;inY++) {
    for (inX=0;inX<SYSVID_WIDTH ;inX++) {
      inColor = gamePal[bmpline[(inY * viswidth) + inX]];
      buffer_scr[inY*SYSVID_WIDTH + inX] = inColor;
    }
  }
}

void bitmap_scale(int viswidth, int visheight, unsigned char *bmpline) {
  uint16_t* buffer_scr = (uint16_t*) gfx_render_target->address;
  unsigned int W,H,ix,iy,x,y;
  x=0;
  y=0;
  W=SYSVID_WIDTH;
  H=SYSVID_HEIGHT;
  ix=(viswidth<<16)/W;
  iy=(visheight<<16)/H;

  do   
  {
    unsigned char *buffer_mem=&bmpline[(y>>16)*viswidth];
    W=SYSVID_WIDTH; x=0;
    do {
      *buffer_scr++=gamePal[buffer_mem[x>>16]];
      x+=ix;
    } while (--W);
    y+=iy;
  } while (--H);
}


int loadFile(char *filename) {
  int ret=0;
  
  FILE* handle = fopen(filename, "rb");  
  if (handle != NULL) {
    fseek(handle, 0, SEEK_END);
    buffer_rom_size = ftell(handle);
    fseek(handle, 0, SEEK_SET);
    buffer_rom = (uInt8*)malloc(buffer_rom_size);
    fread(buffer_rom, 1, buffer_rom_size, handle); 
    fclose(handle);
    ret = 1;
  }

  return ret;
}

int main(int argc, char* argv[]) {
  strcpy(gameName,argv[0]);
  int ref = EXIT_SUCCESS;

  srand(OSTimeGet());

  control_init();

  display* gfx_display = display_create(SYSVID_WIDTH, SYSVID_HEIGHT, SYSVID_WIDTH, (DISPLAY_FORMAT_RGB565 | DISPLAY_STRETCH), NULL, NULL);

  gfx_init(gfx_display);
  gfx_render_target_clear(gfx_color_rgb(0x00, 0x00, 0x00));
  
  gfx_render_target_swap();
  control_lock(TIMER_RESOLUTION / 4);

  gameTimer   = timer_create();

  a320_sound_init();
    
  m_Flag = GF_GAMEINIT;
  //system_loadcfg("A:\\GAME\\StelGOO.CFG");
  GameConf.sndLevel = 40;
  waveout_set_volume(GameConf.sndLevel>>1);

  GameConf.m_ScreenRatio = 1; // force it
  render_screen = GameConf.m_ScreenRatio == 1 ? bitmap_scale : bitmap_noscale;

  uint32_t tempTick  = 0;
  uint32_t tempPauseOS = 0;
  uint32_t frameskip = 0;
  
  int sysref;
		  
  while (m_Flag != GF_GAMEQUIT) {
    sysref = _sys_judge_event(NULL);
    if (sysref < 0) {
      break;  
    }
  
    switch (m_Flag) {
      case GF_MAINUI:
        a320_sound_pause(1);
        //screen_showmenu();
        render_screen = GameConf.m_ScreenRatio == 1 ? bitmap_scale : bitmap_noscale;
        a320_sound_pause(0);
        break;

      case GF_GAMEINIT:
        if (loadFile(gameName)) {
          m_Flag = GF_GAMEQUIT;
          theSound = new SoundSDL(160);
          theSound->setVolume(100);
          theConsole = new Console(buffer_rom, buffer_rom_size, gameName, *theSound);
          /*gamePalette = theConsole->myMediaSource->palette();
          for (uInt32 i = 0; i < 256; i++)	{
            uInt8 r, g, b;
            r = (uInt8) ((gamePalette[i] & 0x00ff0000) >> 16);
            g = (uInt8) ((gamePalette[i] & 0x0000ff00) >> 8);
            b = (uInt8) ((gamePalette[i] & 0x000000ff) );
            gamePal[i]=gfx_color_rgb(r,g,b);
          }*/
          //m_Flag = GF_GAMERUNNING;
        }
        else
          m_Flag = GF_GAMEQUIT;
        break;
		
      case GF_GAMERUNNING:
        tempTick = timer_delta(gameTimer);
        if (tempTick < gameTickRate) {
          tempPauseOS = (((gameTickRate - tempTick) * OS_TICKS_PER_SEC) / TIMER_RESOLUTION);
          if(tempPauseOS > 0) OSTimeDly(tempPauseOS);
        }
        
        theConsole->update();
        render_screen(theConsole->myMediaSource->width(),theConsole->myMediaSource->height(),theConsole->myMediaSource->currentFrameBuffer());
        //render_screen(SYSVID_WIDTH,200,theConsole->myMediaSource->currentFrameBuffer());
        gfx_render_target_swap();
   
        KEY_STATUS key_curr;
        _kbd_get_status( &key_curr);

        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_SPACE, key_curr.status & DK_BUTTON_A );
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_UP,    key_curr.status & DK_DPAD_UP);
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_DOWN,  key_curr.status & DK_DPAD_DOWN);
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_LEFT,  key_curr.status & DK_DPAD_LEFT);
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_RIGHT, key_curr.status & DK_DPAD_RIGHT);
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_F1,    key_curr.status & DK_TRIGGER_RIGHT);
        theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_F2,    key_curr.status & DK_TRIGGER_RIGHT);
        if ( (key_curr.status & DK_BUTTON_START) && (key_curr.status & DK_BUTTON_SELECT) ) {
          m_Flag = GF_MAINUI;
        }
        else
          theConsole->eventHandler().sendKeyEvent(StellaEvent::KCODE_1, key_curr.status & DK_BUTTON_SELECT);
        /*int bt=0;
		while (bt==0) { control_poll(); if (control_check(DK_BUTTON_A).pressed  && control_check(DK_BUTTON_A).changed  ) bt=1; }*/

		/*
		if (key_curr.status & DK_TRIGGER_RIGHT)  {
		  static int aa = 0;
		  char sz[64];
		  aa++;
		  sprintf(sz,"%s.tga",game_name);
		  gfx_texture toto;
		  toto.address = (unsigned short *)(gfx_render_target->address);
		  toto.width = SYSVID_WIDTH;
		  toto.height = SYSVID_HEIGHT;
		  gfx_tex_save_tga(sz, &toto) ;
		}
		*/
 	    break;
    }
  }
  
  a320_sound_stop();
	
  timer_delete(gameTimer);
  
  gfx_term();
  display_delete(gfx_display);
  control_term();
  
  return ref;
}
