#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <dingkit/ucos2.h>
#include <dingkit/audio.h>

#include "a320_snd.h"

waveout_inst *_audio_music = NULL;
waveout_args tempArgs = { A320SAMPLE_RATE, 16, 1, 20 };
  
#define  TASK_START_PRIO 5
#define  TASK_STK_SIZE   1024                      // Size of each task's stacks (# of WORDs)
#define  NO_TASKS        1                         // Number of identical tasks 

OS_STK   TaskStk[NO_TASKS][TASK_STK_SIZE];          // Tasks stacks 
OS_STK   TaskStartStk[TASK_STK_SIZE];
char     TaskData[NO_TASKS];                        // Parameters to pass to each task



static volatile int a320_sound_pausei;
static volatile int a320_audio_done;
static volatile int a320_sound_thread_exit;			  // Flag to end wiz_sound_thread_play() thread
volatile int a320_curSound;

inline int a320_getCurSound()
{
  int curSnd=a320_curSound;  
  curSnd--; if (curSnd<0) curSnd+=16;
  return curSnd;
}

static signed short a320_sound_buf[SNDLENGTH];  // Sound buffer
signed short a320_sound_emu[SNDLENGTH];  // Sound buffer

void a320_sound_pause(int yes) { a320_sound_pausei=yes; }

void a320_new_sound_play(void *none) {
  do { 
    if (a320_sound_pausei == 0) {
      int k;
      for (k = 0; k < SNDLENGTH; k++) {
        a320_sound_buf[k] = a320_sound_emu[k];
      }
      waveout_write(_audio_music, (char*)a320_sound_buf, SNDLENGTH); 
      a320_audio_done=1;
		} 
    else
      OSTimeDly(1);
	} while(a320_sound_thread_exit == 0);							                    // Until the end of the sound thread
  
	a320_sound_thread_exit=0;

  OSTaskDel(TASK_START_PRIO);
}

void a320_sound_thread_mute(void)
{
	//memset(a320_sound_buf_tot,0,(SNDLENGTH*16*2));
}

void a320_sound_thread_stop(void)
{
  a320_sound_pausei = 1;
	a320_sound_thread_exit=1;
	
	a320_sound_thread_mute();
}

void a320_sound_wait(void)
{
	while (!a320_audio_done)
		OSTimeDly(4);
  a320_audio_done = 0;
}


void a320_sound_init(void) {
  a320_sound_thread_exit=0;
  a320_sound_pausei = 1;
  a320_curSound = 0;
  a320_audio_done=1;

  _audio_music = waveout_open(&tempArgs);
	
 a320_sound_thread_mute();
	
  OSTaskCreate(a320_new_sound_play, (void *) 0, (void *)&TaskStartStk[TASK_STK_SIZE - 1], TASK_START_PRIO);
}

void a320_sound_stop(void) {
  a320_sound_thread_stop();

  while (a320_sound_thread_exit);  // to avoid pb with waveout_close -> can hang ??

  waveout_close(_audio_music);
}
